#version 150

#moj_import <fog.glsl>

uniform sampler2D Sampler0;
uniform sampler2D Sampler1;

uniform vec4 ColorModulator;
uniform float FogStart;
uniform float FogEnd;
uniform vec4 FogColor;
uniform float GameTime;

flat in int skySpeed;
in float vertexDistance;
in vec4 vertexColor;
in vec2 texCoord0;
in vec2 texCoord1;
in vec4 normal;

out vec4 fragColor;

#moj_import <nlz_utils.glsl>
#moj_import <nlz_ab_speeds.glsl>


#define TEST 0

#define STARS_SPEED 600

#define MSTARS_SIZE .5
#define MSTARS_SPEED 120.


vec4 mapStars(float time, vec3 view, float bsize, float angleOffset, int skySpeed) {
	vec2 pos = vec2(view.x, view.y);
	bool rev = (view.z > 0);
	if (rev) {
		time = 2*PI - time;
	}


	vec4 color = vec4(vec3(0.), 1.);
	float angle = atan(pos.y, pos.x) + angleOffset;
	float angleNumber = 20.;
	float random = rand(floor(angle * angleNumber) * 100.)* speedParams[skySpeed].w;
	float dist = random;
	dist += time * speedParams[skySpeed].y;
	dist = fract(dist);

	float fragD = length(pos);
	float size = smoothstep(-.1, 2., bsize);
	float bounded = step(dist, fragD) * step(fragD, dist + size);

	bounded += step(fragD, fract(dist + size)) * step(fract(dist + size), dist);    // non continuous


	color.rgb = mix(vec3(0.), vec3(1.), bounded);
	color.r *= 1.;
	color.g *= .7 + (random/speedParams[skySpeed].w) * .3;    // simulate distance
	color.b *= 4.;

	// hard edges
	float angleCenter = abs(fract(angle * angleNumber) * 2. - 1.);
	color.a = step(angleCenter, smoothstep(-.2, 1., bsize));

	// smooth gradient
	color.a *= 1. - angleCenter;
	color.a *= smoothstep(0., 1., fragD);
	color.a *= cos(random * time) * .5 + .5;

	// filter
	if (color.r+color.g+color.b == 0.0)
		color.a = 0.0;

	return color;
}



void main() {
    gl_FragDepth = gl_FragCoord.z;

    vec4 color = texture(Sampler0, texCoord0);
    int a = int(color.a * 255);

    const vec4 skyColor = vec4(0.14, 0, 0.31, 1);

    if (vertexColor.a > 1.) {
        gl_FragDepth *= 1000;
        // fragColor = vec4(normalize(vertexColor.xyz), 1.);
        float t = GameTime * 2 * PI;
        
        vec4 col = mapStars(t, normalize(vertexColor.xyz), speedParams[skySpeed].x, 20., skySpeed);//* .5 + smoothstep(.9, 1., speedParams[skySpeed].x);

        fragColor = mix(skyColor, col, col.a);
        
        
        
        // DEBUG_FRAG(RED_VEC4);
    } else if (color.a < 0.1) {
        discard;
    } else if (a <= 254 && a >= 253) {
        color.a = 1.0;
        fragColor = linear_fog(color, vertexDistance, FogStart, FogEnd, FogColor);
    } else if (a == 252) {
        color.a = 1.0;
        fragColor = linear_fog(color*mix(color, vertexColor, .25), vertexDistance, FogStart, FogEnd, FogColor);
    } else {
        fragColor = linear_fog(color * vertexColor * ColorModulator, vertexDistance, FogStart, FogEnd, FogColor);
    }
}